@@ -14,6 +14,11 @@
 // You should have received a copy of the GNU General Public License
 // along with Moodle.  If not, see <http://www.gnu.org/licenses/>.
 
+namespace enrol_meta;
+
+use context_course;
+use enrol_meta_plugin;
+
 /**
  * Meta enrolment sync functional test.
  *
@@ -22,12 +27,7 @@
  * @copyright  2013 Petr Skoda {@link http://skodak.org}
  * @license    http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
  */
-
-defined('MOODLE_INTERNAL') || die();
-
-global $CFG;
-
-class enrol_meta_plugin_testcase extends advanced_testcase {
+class plugin_test extends \advanced_testcase {
 
     protected function enable_plugin() {
         $enabled = enrol_get_plugins(true);
@@ -60,7 +60,7 @@ class enrol_meta_plugin_testcase extends advanced_testcase {
     protected function has_role($user, $enrol, $role) {
         global $DB;
 
-        $context = context_course::instance($enrol->courseid);
+        $context = \context_course::instance($enrol->courseid);
 
         if ($role === false) {
             if ($DB->record_exists('role_assignments', array('contextid'=>$context->id, 'userid'=>$user->id, 'component'=>'enrol_meta', 'itemid'=>$enrol->id))) {
@@ -294,7 +294,7 @@ class enrol_meta_plugin_testcase extends advanced_testcase {
         $this->assertEquals(10, $DB->count_records('user_enrolments', array('status'=>ENROL_USER_ACTIVE)));
         $this->assertFalse($this->is_meta_enrolled($user1, $enrol1, $student));
 
-        role_assign($teacher->id, $user1->id, context_course::instance($course1->id)->id);
+        role_assign($teacher->id, $user1->id, \context_course::instance($course1->id)->id);
         $this->assertEquals(11, $DB->count_records('user_enrolments'));
         $this->assertEquals(10, $DB->count_records('role_assignments'));
         $this->assertEquals(11, $DB->count_records('user_enrolments', array('status'=>ENROL_USER_ACTIVE)));
@@ -305,7 +305,7 @@ class enrol_meta_plugin_testcase extends advanced_testcase {
         $this->assertEquals(11, $DB->count_records('user_enrolments', array('status'=>ENROL_USER_ACTIVE)));
         $this->assertTrue($this->is_meta_enrolled($user1, $enrol1, $teacher));
 
-        role_unassign($teacher->id, $user1->id, context_course::instance($course1->id)->id);
+        role_unassign($teacher->id, $user1->id, \context_course::instance($course1->id)->id);
         $this->assertEquals(10, $DB->count_records('user_enrolments'));
         $this->assertEquals(8, $DB->count_records('role_assignments'));
         $this->assertEquals(10, $DB->count_records('user_enrolments', array('status'=>ENROL_USER_ACTIVE)));
@@ -510,29 +510,29 @@ class enrol_meta_plugin_testcase extends advanced_testcase {
         enrol_get_plugin('manual')->unenrol_user($manualenrol1, $user1->id);
         $this->assertFalse(groups_is_member($group31->id, $user1->id));
         $this->assertTrue(groups_is_member($group32->id, $user1->id));
-        $this->assertTrue(is_enrolled(context_course::instance($course3->id), $user1, '', true)); // He still has active enrolment.
+        $this->assertTrue(is_enrolled(\context_course::instance($course3->id), $user1, '', true)); // He still has active enrolment.
         // And the same after sync.
         enrol_meta_sync(null, false);
         $this->assertFalse(groups_is_member($group31->id, $user1->id));
         $this->assertTrue(groups_is_member($group32->id, $user1->id));
-        $this->assertTrue(is_enrolled(context_course::instance($course3->id), $user1, '', true));
+        $this->assertTrue(is_enrolled(\context_course::instance($course3->id), $user1, '', true));
 
         // Unenroll user1 from course2 and make sure he is completely unenrolled from course3.
         enrol_get_plugin('manual')->unenrol_user($manualenrol2, $user1->id);
         $this->assertFalse(groups_is_member($group32->id, $user1->id));
-        $this->assertFalse(is_enrolled(context_course::instance($course3->id), $user1));
+        $this->assertFalse(is_enrolled(\context_course::instance($course3->id), $user1));
 
         set_config('unenrolaction', ENROL_EXT_REMOVED_SUSPENDNOROLES, 'enrol_meta');
 
         // When user is unenrolled in this case, he is still a member of a group (but enrolment is suspended).
         enrol_get_plugin('manual')->unenrol_user($manualenrol1, $user4->id);
         $this->assertTrue(groups_is_member($group31->id, $user4->id));
-        $this->assertTrue(is_enrolled(context_course::instance($course3->id), $user4));
-        $this->assertFalse(is_enrolled(context_course::instance($course3->id), $user4, '', true));
+        $this->assertTrue(is_enrolled(\context_course::instance($course3->id), $user4));
+        $this->assertFalse(is_enrolled(\context_course::instance($course3->id), $user4, '', true));
         enrol_meta_sync(null, false);
         $this->assertTrue(groups_is_member($group31->id, $user4->id));
-        $this->assertTrue(is_enrolled(context_course::instance($course3->id), $user4));
-        $this->assertFalse(is_enrolled(context_course::instance($course3->id), $user4, '', true));
+        $this->assertTrue(is_enrolled(\context_course::instance($course3->id), $user4));
+        $this->assertFalse(is_enrolled(\context_course::instance($course3->id), $user4, '', true));
     }
 
     /**
@@ -597,6 +597,40 @@ class enrol_meta_plugin_testcase extends advanced_testcase {
 
     }
 
+    /**
+     * Test enrolling users in a course, where the customint2 (group) property of the instance points to an invalid group
+     *
+     * @covers \enrol_meta_handler::sync_with_parent_course
+     * @covers ::enrol_meta_sync
+     */
+    public function test_add_to_group_invalid(): void {
+        $this->resetAfterTest();
+
+        $this->enable_plugin();
+
+        $courseone = $this->getDataGenerator()->create_course();
+        $coursetwo = $this->getDataGenerator()->create_course();
+
+        /** @var enrol_meta_plugin $plugin */
+        $plugin = enrol_get_plugin('meta');
+        $plugin->add_instance($coursetwo, ['customint1' => $courseone->id, 'customint2' => 42]);
+
+        // Ensure the event observer works for invalid groups.
+        $userone = $this->getDataGenerator()->create_and_enrol($courseone);
+
+        // Now disable the plugin, add another enrolment.
+        $this->disable_plugin();
+        $usertwo = $this->getDataGenerator()->create_and_enrol($courseone);
+
+        // Re-enable the plugin, run sync task - should also work for invalid groups.
+        $this->enable_plugin();
+        enrol_meta_sync($coursetwo->id);
+
+        $coursetwocontext = context_course::instance($coursetwo->id);
+        $this->assertTrue(is_enrolled($coursetwocontext, $userone));
+        $this->assertTrue(is_enrolled($coursetwocontext, $usertwo));
+    }
+
     /**
      * Test user_enrolment_created event.
      */
@@ -626,11 +660,6 @@ class enrol_meta_plugin_testcase extends advanced_testcase {
         $dbuserenrolled = $DB->get_record('user_enrolments', array('userid' => $user1->id));
         $this->assertInstanceOf('\core\event\user_enrolment_created', $event);
         $this->assertEquals($dbuserenrolled->id, $event->objectid);
-        $this->assertEquals('user_enrolled', $event->get_legacy_eventname());
-        $expectedlegacyeventdata = $dbuserenrolled;
-        $expectedlegacyeventdata->enrol = 'meta';
-        $expectedlegacyeventdata->courseid = $course2->id;
-        $this->assertEventLegacyData($expectedlegacyeventdata, $event);
         $this->assertEventContextNotUsed($event);
     }
 
@@ -664,7 +693,6 @@ class enrol_meta_plugin_testcase extends advanced_testcase {
 
         $this->assertEquals(0, $DB->count_records('user_enrolments'));
         $this->assertInstanceOf('\core\event\user_enrolment_deleted', $event);
-        $this->assertEquals('user_unenrolled', $event->get_legacy_eventname());
         $this->assertEventContextNotUsed($event);
     }
 
@@ -700,13 +728,6 @@ class enrol_meta_plugin_testcase extends advanced_testcase {
         $dbuserenrolled = $DB->get_record('user_enrolments', array('userid' => $user1->id));
         $this->assertInstanceOf('\core\event\user_enrolment_updated', $event);
         $this->assertEquals($dbuserenrolled->id, $event->objectid);
-        $this->assertEquals('user_enrol_modified', $event->get_legacy_eventname());
-        $expectedlegacyeventdata = $dbuserenrolled;
-        $expectedlegacyeventdata->enrol = 'meta';
-        $expectedlegacyeventdata->courseid = $course2->id;
-        $url = new \moodle_url('/enrol/editenrolment.php', array('ue' => $event->objectid));
-        $this->assertEquals($url, $event->get_url());
-        $this->assertEventLegacyData($expectedlegacyeventdata, $event);
         $this->assertEventContextNotUsed($event);
     }
 
@@ -714,12 +735,15 @@ class enrol_meta_plugin_testcase extends advanced_testcase {
      * Test that a new group with the name of the course is created.
      */
     public function test_enrol_meta_create_new_group() {
-        global $DB;
+        global $DB, $CFG;
         $this->resetAfterTest();
         // Create two courses.
         $course = $this->getDataGenerator()->create_course(array('fullname' => 'Mathematics'));
         $course2 = $this->getDataGenerator()->create_course(array('fullname' => 'Physics'));
         $metacourse = $this->getDataGenerator()->create_course(array('fullname' => 'All sciences'));
+
+        require_once($CFG->dirroot.'/enrol/meta/locallib.php');
+
         // Run the function.
         $groupid = enrol_meta_create_new_group($metacourse->id, $course->id);
         // Check the results.
@@ -730,7 +754,7 @@ class enrol_meta_plugin_testcase extends advanced_testcase {
         $this->assertEquals($metacourse->id, $group->courseid);
 
         // Create a group that will have the same name as the course.
-        $groupdata = new stdClass();
+        $groupdata = new \stdClass();
         $groupdata->courseid = $metacourse->id;
         $groupdata->name = 'Physics course';
         groups_create_group($groupdata);
@@ -870,4 +894,199 @@ class enrol_meta_plugin_testcase extends advanced_testcase {
         $this->assertEquals($expectedenrolments, $enrolments);
         $sink->close();
     }
+
+    /**
+     * Test for getting user enrolment actions.
+     */
+    public function test_get_user_enrolment_actions() {
+        global $CFG, $PAGE;
+        $this->resetAfterTest();
+
+        // Set page URL to prevent debugging messages.
+        $PAGE->set_url('/enrol/editinstance.php');
+
+        $pluginname = 'meta';
+
+        // Only enable the meta enrol plugin.
+        $CFG->enrol_plugins_enabled = $pluginname;
+
+        $generator = $this->getDataGenerator();
+
+        // Get the enrol plugin.
+        $plugin = enrol_get_plugin($pluginname);
+
+        // Create a course.
+        $course = $generator->create_course();
+        // Enable this enrol plugin for the course.
+        $plugin->add_instance($course);
+
+        // Create a student.
+        $student = $generator->create_user();
+        // Enrol the student to the course.
+        $generator->enrol_user($student->id, $course->id, 'student', $pluginname);
+
+        // Teachers don't have enrol/meta:unenrol capability by default. Login as admin for simplicity.
+        $this->setAdminUser();
+        require_once($CFG->dirroot . '/enrol/locallib.php');
+        $manager = new \course_enrolment_manager($PAGE, $course);
+
+        $userenrolments = $manager->get_user_enrolments($student->id);
+        $this->assertCount(1, $userenrolments);
+
+        $ue = reset($userenrolments);
+        $actions = $plugin->get_user_enrolment_actions($manager, $ue);
+        // Meta-link enrolment has no enrol actions for active students.
+        $this->assertCount(0, $actions);
+
+        // Enrol actions for a suspended student.
+        // Suspend the student.
+        $ue->status = ENROL_USER_SUSPENDED;
+
+        $actions = $plugin->get_user_enrolment_actions($manager, $ue);
+        // Meta-link enrolment has enrol actions for suspended students -- unenrol.
+        $this->assertCount(1, $actions);
+    }
+
+    /**
+     * Test how data for instance editing is validated.
+     */
+    public function test_edit_instance_validation() {
+        global $DB;
+
+        $this->resetAfterTest();
+
+        $metaplugin = enrol_get_plugin('meta');
+
+        // A course with meta enrolment.
+        $course = $this->getDataGenerator()->create_course();
+        $coursecontext = \context_course::instance($course->id);
+
+        // Create a meta enrolment instance.
+        $instance = (object)$metaplugin->get_instance_defaults();
+        $instance->id       = null;
+        $instance->courseid = $course->id;
+        $instance->status   = ENROL_INSTANCE_ENABLED;
+        // Emulate the form data.
+        $data = [
+            'customint1' => 0,
+            'customint2' => 0
+        ];
+        // Test when no valid 'customint1' field (meta courses links) is provided.
+        $errors = $metaplugin->edit_instance_validation($data, [], $instance, $coursecontext);
+        // We're going to check the string contents of the errors returned as this is the only way
+        // to differentiate the errors produced by the 'edit_instance_validation()' method somehow.
+        // The method always returns what the edit instance form expects and this is an array of form fields
+        // with the corresponding errors messages.
+        $this->assertEquals('Required', $errors['customint1']);
+
+        // Test when 'customint1' contains an unknown course.
+        // Fetch the max course id from the courses table and increment it to get
+        // the course id which surely doesn't exist.
+        $maxid = $DB->get_field_sql('SELECT MAX(id) FROM {course}');
+        // Use the same instance as before but set another data.
+        $data = [
+            'customint1' => [$maxid + 1],
+            'customint2' => 0
+        ];
+        $errors = $metaplugin->edit_instance_validation($data, [], $instance, $coursecontext);
+        $this->assertEquals('You are trying to use an invalid course ID', $errors['customint1']);
+
+        // Test when 'customint1' field already contains courses meta linked with the current one.
+        $metacourse1 = $this->getDataGenerator()->create_course();
+        $metaplugin->add_instance($course, array('customint1' => $metacourse1->id));
+        // Use the same instance as before but set another data.
+        $data = [
+            'customint1' => [$metacourse1->id],
+            'customint2' => 0
+        ];
+        $errors = $metaplugin->edit_instance_validation($data, [], $instance, $coursecontext);
+        $this->assertEquals('You are trying to use an invalid course ID', $errors['customint1']);
+
+        // Test when a course is set as a not visible and a user doesn't have the capability to use it here.
+        $metacourse2record = new \stdClass();
+        $metacourse2record->visible = 0;
+        $metacourse2 = $this->getDataGenerator()->create_course($metacourse2record);
+        $metacourse2context = \context_course::instance($metacourse2->id);
+
+        $user = $this->getDataGenerator()->create_user();
+        $teacherrole = $DB->get_record('role', array('shortname' => 'teacher'));
+        role_assign($teacherrole->id, $user->id, $metacourse2context->id);
+        unassign_capability('moodle/course:viewhiddencourses', $teacherrole->id);
+        $this->setUser($user);
+
+        // Use the same instance as before but set another data.
+        $data = [
+            'customint1' => [$metacourse2->id],
+            'customint2' => 0
+        ];
+        $errors = $metaplugin->edit_instance_validation($data, [], $instance, $coursecontext);
+        $this->assertEquals('Sorry, but you do not currently have permissions to do that (moodle/course:viewhiddencourses).',
+            $errors['customint1']);
+
+        // Revert some changes from the last assertion to reuse the course.
+        $metacourse2->visible = 1;
+        $DB->update_record('course', $metacourse2);
+        assign_capability('moodle/course:viewhiddencourses', CAP_ALLOW,
+            $teacherrole->id, \context_course::instance($metacourse2->id));
+
+        // Test with no 'enrol/meta:selectaslinked' capability.
+        unassign_capability('enrol/meta:selectaslinked', $teacherrole->id);
+        $errors = $metaplugin->edit_instance_validation($data, [], $instance, $coursecontext);
+        $this->assertEquals('Sorry, but you do not currently have permissions to do that (enrol/meta:selectaslinked).',
+            $errors['customint1']);
+
+        // Back to admin user to regain the capabilities quickly.
+        $this->setAdminUser();
+
+        // Test when meta course id is the site id.
+        $site = $DB->get_record('course', ['id' => SITEID]);
+        // Use the same instance as before but set another data.
+        $data = [
+            'customint1' => [$site->id],
+            'customint2' => 0
+        ];
+        $errors = $metaplugin->edit_instance_validation($data, [], $instance, $coursecontext);
+        $this->assertEquals('You are trying to use an invalid course ID', $errors['customint1']);
+
+        // Test when meta course id is id of the current course.
+        // Use the same instance as before but set another data.
+        $data = [
+            'customint1' => [$course->id],
+            'customint2' => 0
+        ];
+        $errors = $metaplugin->edit_instance_validation($data, [], $instance, $coursecontext);
+        $this->assertEquals('You are trying to use an invalid course ID', $errors['customint1']);
+
+        // Test with the 'customint2' field set (which is groups).
+        // Prepare some groups data.
+        $this->getDataGenerator()->create_group(array('courseid' => $course->id));
+        $this->getDataGenerator()->create_group(array('courseid' => $course->id));
+        $groups = [];
+        foreach (groups_get_all_groups($course->id) as $group) {
+            $groups[$group->id] = format_string($group->name, true, array('context' => $coursecontext));
+        }
+
+        // Use the same instance as before but set another data.
+        // Use a non-existing group id.
+        if (!$maxid = $DB->get_field_sql('SELECT MAX(id) FROM {groups}')) {
+            $maxid = 0;
+        }
+        $data = [
+            'customint1' => [$metacourse2->id],
+            'customint2' => [$maxid + 1]
+        ];
+        $errors = $metaplugin->edit_instance_validation($data, [], $instance, $coursecontext);
+        $this->assertArrayHasKey('customint2', $errors);
+
+        // Test with valid data.
+        reset($groups);
+        $validgroup = key($groups);
+        $data = [
+            'customint1' => [$metacourse2->id],
+            'customint2' => $validgroup
+        ];
+        $errors = $metaplugin->edit_instance_validation($data, [], $instance, $coursecontext);
+        $this->assertArrayNotHasKey('customint1', $errors);
+        $this->assertArrayNotHasKey('customint2', $errors);
+    }
 }
