@@ -18,9 +18,11 @@
  * core_component related tests.
  *
  * @package    core
- * @category   phpunit
+ * @category   test
  * @copyright  2013 Petr Skoda {@link http://skodak.org}
  * @license    http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
+ *
+ * @covers \core_component
  */
 final class component_test extends advanced_testcase {
     /**
@@ -28,25 +30,11 @@ final class component_test extends advanced_testcase {
      * this is defined here to annoy devs that try to add more without any thinking,
      * always verify that it does not collide with any existing add-on modules and subplugins!!!
      */
-    const SUBSYSTEMCOUNT = 75;
+    const SUBSYSTEMCOUNT = 77;
 
-    public function setUp(): void {
-        $psr0namespaces = new ReflectionProperty('core_component', 'psr0namespaces');
-        $psr0namespaces->setAccessible(true);
-        $this->oldpsr0namespaces = $psr0namespaces->getValue(null);
-
-        $psr4namespaces = new ReflectionProperty('core_component', 'psr4namespaces');
-        $psr4namespaces->setAccessible(true);
-        $this->oldpsr4namespaces = $psr4namespaces->getValue(null);
-    }
+    #[\Override]
     public function tearDown(): void {
-        $psr0namespaces = new ReflectionProperty(\core_component::class, 'psr0namespaces');
-        $psr0namespaces->setAccessible(true);
-        $psr0namespaces->setValue(null, $this->oldpsr0namespaces);
-
-        $psr4namespaces = new ReflectionProperty(\core_component::class, 'psr4namespaces');
-        $psr4namespaces->setAccessible(true);
-        $psr4namespaces->setValue(null, $this->oldpsr4namespaces);
+        parent::tearDown();
 
         \core_component::reset();
     }
@@ -404,7 +392,7 @@ final class component_test extends advanced_testcase {
      */
     public function test_get_component_from_classname(
         string $classname,
-        $expected
+        string|null $expected,
     ): void {
         $this->assertEquals(
             $expected,
@@ -588,7 +576,7 @@ final class component_test extends advanced_testcase {
      */
     public function test_get_component_classes_in_namespace_provider(
         array $methodargs,
-        string $expectedclassnameformat
+        string $expectedclassnameformat,
     ): void {
         $classlist = core_component::get_component_classes_in_namespace(...$methodargs);
         $this->assertGreaterThan(0, count($classlist));
@@ -773,6 +761,7 @@ final class component_test extends advanced_testcase {
      * @param array $psr4 The PSR-4 namespaces to be used in the test.
      * @param string $classname The name of the class to attempt to load.
      * @param string $includedfiles The file expected to be loaded.
+     * @runInSeparateProcess
      */
     public function test_classloader($psr0, $psr4, $classname, $includedfiles) {
         $psr0namespaces = new ReflectionProperty('core_component', 'psr0namespaces');
@@ -802,7 +791,8 @@ final class component_test extends advanced_testcase {
         // This is not in the spec, but may come up in some libraries using both namespaces and PEAR-style class names.
         // If problems arise we can remove this test, but will need to add a warning.
         // Normalise to forward slash for testing purposes.
-        $directory = str_replace('\\', '/', $CFG->dirroot) . "/lib/tests/fixtures/component/";
+        $dirroot = str_replace('\\', '/', $CFG->dirroot);
+        $directory = "{$dirroot}/lib/tests/fixtures/component/";
 
         $psr0 = [
           'psr0'      => 'lib/tests/fixtures/component/psr0',
@@ -873,6 +863,28 @@ final class component_test extends advanced_testcase {
                 'classname' => 'overlap_subnamespace_example2',
                 'file' => "{$directory}overlap/subnamespace/example2.php",
             ],
+            'PSR-4 namespaces can come from multiple sources - first source' => [
+                'psr0' => $psr0,
+                'psr4' => [
+                    'Psr\\Http\\Message' => [
+                        'lib/psr/http-message/src',
+                        'lib/psr/http-factory/src',
+                    ],
+                ],
+                'classname' => 'Psr\Http\Message\ServerRequestInterface',
+                'includedfiles' => "{$dirroot}/lib/psr/http-message/src/ServerRequestInterface.php",
+            ],
+            'PSR-4 namespaces can come from multiple sources - second source' => [
+                'psr0' => [],
+                'psr4' => [
+                    'Psr\\Http\\Message' => [
+                        'lib/psr/http-message/src',
+                        'lib/psr/http-factory/src',
+                    ],
+                ],
+                'classname' => 'Psr\Http\Message\ServerRequestFactoryInterface',
+                'includedfiles' => "{$dirroot}/lib/psr/http-factory/src/ServerRequestFactoryInterface.php",
+            ],
         ];
     }
 
@@ -942,6 +954,7 @@ final class component_test extends advanced_testcase {
      * @param array $psr4 The PSR-4 namespaces to be used in the test.
      * @param string $classname The name of the class to attempt to load.
      * @param string|bool $file The expected file corresponding to the class or false for nonexistant.
+     * @runInSeparateProcess
      */
     public function test_psr_classloader($psr0, $psr4, $classname, $file) {
         $psr0namespaces = new ReflectionProperty('core_component', 'psr0namespaces');
@@ -1057,7 +1070,7 @@ final class component_test extends advanced_testcase {
      */
     public function test_get_component_names(
         bool $includecore,
-        bool $coreexpected
+        bool $coreexpected,
     ): void {
         global $CFG;
         $componentnames = \core_component::get_component_names($includecore);
@@ -1101,6 +1114,74 @@ final class component_test extends advanced_testcase {
         ];
     }
 
+    /**
+     * Basic tests for APIs related functions in the core_component class.
+     */
+    public function test_apis_methods() {
+        $apis = core_component::get_core_apis();
+        $this->assertIsArray($apis);
+
+        $apinames = core_component::get_core_api_names();
+        $this->assertIsArray($apis);
+
+        // Both should return the very same APIs.
+        $this->assertEquals($apinames, array_keys($apis));
+
+        $this->assertFalse(core_component::is_core_api('lalala'));
+        $this->assertTrue(core_component::is_core_api('privacy'));
+    }
+
+    /**
+     * Test that the apis.json structure matches expectations
+     *
+     * While we include an apis.schema.json file in core, there isn't any PHP built-in allowing us
+     * to validate it (3rd part libraries needed). Plus the schema doesn't allow to validate things
+     * like uniqueness or sorting. We are going to do all that here.
+     */
+    public function test_apis_json_validation() {
+        $apis = $sortedapis = core_component::get_core_apis();
+        ksort($sortedapis); // We'll need this later.
+
+        $subsystems = core_component::get_core_subsystems(); // To verify all apis are pointing to valid subsystems.
+        $subsystems['core'] = 'anything'; // Let's add 'core' because it's a valid component for apis.
+
+        // General structure validations.
+        $this->assertIsArray($apis);
+        $this->assertGreaterThan(25, count($apis));
+        $this->assertArrayHasKey('privacy', $apis); // Verify a few.
+        $this->assertArrayHasKey('external', $apis);
+        $this->assertArrayHasKey('search', $apis);
+        $this->assertEquals(array_keys($sortedapis), array_keys($apis)); // Verify json is sorted alphabetically.
+
+        // Iterate over all apis and perform more validations.
+        foreach ($apis as $apiname => $attributes) {
+            // Message, to be used later and easier finding the problem.
+            $message = "Validation problem found with API: {$apiname}";
+
+            $this->assertIsObject($attributes, $message);
+            $this->assertMatchesRegularExpression('/^[a-z][a-z0-9]+$/', $apiname, $message);
+            $this->assertEquals(['component', 'allowedlevel2', 'allowedspread'], array_keys((array)$attributes), $message);
+
+            // Verify attributes.
+            if ($apiname !== 'core') { // Exception for core api, it doesn't have component.
+                // Check that component attribute looks correct.
+                $this->assertMatchesRegularExpression('/^(core|[a-z][a-z0-9_]+)$/', $attributes->component, $message);
+                // Ensure that the api component (without the core_ prefix) is a correct subsystem.
+                $this->assertArrayHasKey(str_replace('core_', '', $attributes->component), $subsystems, $message);
+            } else {
+                $this->assertNull($attributes->component, $message);
+            }
+
+
+            // Now check for the rest of attributes.
+            $this->assertIsBool($attributes->allowedlevel2, $message);
+            $this->assertIsBool($attributes->allowedspread, $message);
+
+            // Cannot spread if level2 is not allowed.
+            $this->assertLessThanOrEqual($attributes->allowedlevel2, $attributes->allowedspread, $message);
+        }
+    }
+
     /**
      * Test for monologo icons check in plugins.
      *
@@ -1115,4 +1196,35 @@ final class component_test extends advanced_testcase {
         // The function will return false for a non-existent component.
         $this->assertFalse(core_component::has_monologo_icon('randomcomponent', 'h5p'));
     }
+
+    /*
+     * Tests the getter for the db directory summary hash.
+     *
+     * @covers \core_component::get_all_directory_hashes
+     */
+    public function test_get_db_directories_hash() {
+        $initial = \core_component::get_all_component_hash();
+
+        $dir = make_request_directory();
+        $hashes = \core_component::get_all_directory_hashes([$dir]);
+        $emptydirhash = \core_component::get_all_component_hash([$hashes]);
+
+        // Confirm that a single empty directory is a different hash to the core hash.
+        $this->assertNotEquals($initial, $emptydirhash);
+
+        // Now lets add something to the dir, and check the hash is different.
+        $file = fopen($dir . '/test.php', 'w');
+        fwrite($file, 'sometestdata');
+        fclose($file);
+
+        $hashes = \core_component::get_all_directory_hashes([$dir]);
+        $onefiledirhash = \core_component::get_all_component_hash([$hashes]);
+        $this->assertNotEquals($emptydirhash, $onefiledirhash);
+
+        // Now add a subdirectory inside the request dir. This should not affect the hash.
+        mkdir($dir . '/subdir');
+        $hashes = \core_component::get_all_directory_hashes([$dir]);
+        $finalhash = \core_component::get_all_component_hash([$hashes]);
+        $this->assertEquals($onefiledirhash, $finalhash);
+    }
 }
