@@ -32,6 +32,7 @@ $string['addaquestion'] = 'a new question';
 $string['addasection'] = 'a new section heading';
 $string['addarandomquestion'] = 'a random question';
 $string['addarandomquestion_help'] = 'When a random question is added, it results in a randomly-chosen question from the category being inserted into the quiz. This means that different students are likely to get a different selection of questions, and when a quiz allows multiple attempts then each attempt is likely to contain a new selection of questions.';
+$string['addarandomquestion_success'] = 'Random questions have been added';
 $string['addarandomselectedquestion'] = 'Add a random selected question ...';
 $string['adddescriptionlabel'] = 'Add a description item';
 $string['addingquestion'] = 'Adding a question';
@@ -184,18 +185,17 @@ $string['comments'] = 'Comments';
 $string['completedon'] = 'Completed on';
 $string['completiondetail:minattempts'] = 'Make attempts: {$a}';
 $string['completiondetail:passorexhaust'] = 'Receive a pass grade or complete all available attempts';
-$string['completionminattempts'] = 'Minimum number of attempts:';
+$string['completionminattempts'] = 'Minimum attempts';
 $string['completionminattemptsdesc'] = 'Minimum number of attempts required: {$a}';
-$string['completionminattemptsgroup'] = 'Require attempts';
 $string['completionminattemptserror'] = 'Minimum number of attempts must be lower or equal to attempts allowed.';
 $string['completionpassorattemptsexhausteddesc'] = 'Student must achieve a passing grade, or exhaust all available attempts to complete this activity';
-$string['completionattemptsexhausted'] = 'Or all available attempts completed';
+$string['completionattemptsexhausted'] = 'Passing grade or all available attempts completed';
 $string['completionattemptsexhausted_help'] = 'Mark quiz complete when the student has exhausted the maximum number of attempts.';
 $string['configadaptive'] = 'If you choose Yes for this option then the student will be allowed multiple responses to a question even within the same attempt at the quiz.';
 $string['configattemptsallowed'] = 'Restriction on the number of attempts students are allowed at the quiz.';
 $string['configdecimaldigits'] = 'Number of digits that should be shown after the decimal point when displaying grades.';
 $string['configdecimalplaces'] = 'Number of digits that should be shown after the decimal point when displaying grades for the quiz.';
-$string['configdecimalplacesquestion'] = 'Number of digits that should be shown after the decimal point when displaying the grade for individual questions.';
+$string['configdecimalplacesquestion'] = 'Number of digits that should be shown after the decimal point when displaying the mark for individual questions.';
 $string['configdelaylater'] = 'If you set a time delay here, the student cannot start their third, fourth, ... attempt until this much time has passed since the end of their previous attempt.';
 $string['configdelay1'] = 'If you set a time delay, then a student has to wait for that time before they can attempt a quiz after the first attempt.';
 $string['configdelay1st2nd'] = 'If you set a time delay here, the student cannot start their second attempt until this much time has passed since the end of their first attempt.';
@@ -218,7 +218,7 @@ $string['configshufflewithin'] = 'If you enable this option, then the parts maki
 $string['configtimelimit'] = 'Default time limit for quizzes in minutes. 0 mean no time limit.';
 $string['configtimelimitsec'] = 'Default time limit for quizzes in seconds. 0 mean no time limit.';
 $string['configurerandomquestion'] = 'Configure question';
-$string['confirmclose'] = 'Once you submit, you will no longer be able to change your answers for this attempt.';
+$string['confirmclose'] = 'Once you submit your answers, you won’t be able to change them.';
 $string['confirmremovequestion'] = 'Are you sure you want to remove this {$a} question?';
 $string['confirmremovesectionheading'] = 'Are you sure you want to remove the \'{$a}\' section heading?';
 $string['confirmserverdelete'] = 'Are you sure you want to remove the server <b>{$a}</b> from the list?';
@@ -258,8 +258,8 @@ $string['daysavailable'] = 'Days available';
 $string['decimaldigits'] = 'Decimal digits in grades';
 $string['decimalplaces'] = 'Decimal places in grades';
 $string['decimalplaces_help'] = 'This setting specifies the number of digits shown after the decimal point when displaying grades. It only affects the display of grades, not the grades stored in the database, nor the internal calculations, which are carried out to full accuracy.';
-$string['decimalplacesquestion'] = 'Decimal places in question grades';
-$string['decimalplacesquestion_help'] = 'This setting specifies the number of digits shown after the decimal point when displaying the grades for individual questions.';
+$string['decimalplacesquestion'] = 'Decimal places in marks for questions';
+$string['decimalplacesquestion_help'] = 'The number of digits shown after the decimal point when displaying the marks for individual questions.';
 $string['decimalpoints'] = 'Decimal places';
 $string['default'] = 'Default';
 $string['defaultgrade'] = 'Default question grade';
@@ -284,6 +284,8 @@ $string['dragtostart'] = 'To the start';
 $string['duplicateresponse'] = 'This submission has been ignored because you gave an equivalent answer earlier.';
 $string['eachattemptbuildsonthelast'] = 'Each attempt builds on the last';
 $string['eachattemptbuildsonthelast_help'] = 'If multiple attempts are allowed and this setting is enabled, each new quiz attempt will contain the results of the previous attempt. This allows a quiz to be completed over several attempts.';
+$string['edit_slotdisplaynumber_hint'] = 'Edit question number (maximum 16 characters)';
+$string['edit_slotdisplaynumber_label'] = 'New value for {$a}';
 $string['editcategories'] = 'Edit categories';
 $string['editcategory'] = 'Edit category';
 $string['editcatquestions'] = 'Edit category questions';
@@ -368,10 +370,11 @@ $string['eventpagebreakcreated'] = 'Page break created';
 $string['eventpagebreakdeleted'] = 'Page break deleted';
 $string['eventquestionmanuallygraded'] = 'Question manually graded';
 $string['eventquizattemptabandoned'] = 'Quiz attempt abandoned';
-$string['eventquizattempttimelimitexceeded'] = 'Quiz attempt time limit exceeded';
 $string['eventquizattemptregraded'] = 'Quiz attempt regraded';
+$string['eventquizattemptreopened'] = 'Quiz attempt reopened';
 $string['eventquizattemptstarted'] = 'Quiz attempt started';
 $string['eventquizattemptsubmitted'] = 'Quiz attempt submitted';
+$string['eventquizattempttimelimitexceeded'] = 'Quiz attempt time limit exceeded';
 $string['eventquizgradeupdated'] = 'Quiz grade updated';
 $string['eventquizrepaginated'] = 'Quiz re-paginated';
 $string['eventreportviewed'] = 'Quiz report viewed';
@@ -381,6 +384,7 @@ $string['eventsectiontitleupdated'] = 'Section title updated';
 $string['eventsectionshuffleupdated'] = 'Section shuffle updated';
 $string['eventslotcreated'] = 'Slot created';
 $string['eventslotdeleted'] = 'Slot deleted';
+$string['eventslotdisplayedquestionnumberupdated'] = 'Slot displayed question number updated';
 $string['eventslotmarkupdated'] = 'Slot mark updated';
 $string['eventslotmoved'] = 'Slot moved';
 $string['eventslotrequirepreviousupdated'] = 'Slot require previous updated';
@@ -492,7 +496,7 @@ $string['inprogress'] = 'In progress';
 $string['introduction'] = 'Description';
 $string['invalidattemptid'] = 'No such attempt ID exists';
 $string['invalidcategory'] = 'Category ID is invalid';
-$string['invalidoverrideid'] = 'Invalid override id';
+$string['invalidoverrideid'] = 'Invalid override ID.';
 $string['invalidquestionid'] = 'Invalid question id';
 $string['invalidquizid'] = 'Invalid quiz ID';
 $string['invalidrandomslot'] = 'Invalid random question slot id.';
@@ -505,8 +509,6 @@ $string['layoutasshown'] = 'Page layout as shown.';
 $string['layoutasshownwithpages'] = 'Page layout as shown. <small>(Automatic new page every {$a} questions.)</small>';
 $string['layoutshuffledandpaged'] = 'Questions randomly shuffled with {$a} questions per page.';
 $string['layoutshuffledsinglepage'] = 'Questions randomly shuffled, all on one page.';
-$string['legacyquizaccessrulescron'] = 'Legacy cron quiz access rules';
-$string['legacyquizreportscron'] = 'Legacy cron quiz reports';
 $string['link'] = 'Link';
 $string['listitems'] = 'Listing of items in quiz';
 $string['literal'] = 'Literal';
@@ -518,7 +520,7 @@ $string['manualgradequestion'] = 'Manually grade question {$a->question} in {$a-
 $string['mark'] = 'Submit';
 $string['markall'] = 'Submit page';
 $string['marks'] = 'Marks';
-$string['marks_help'] = 'The numerical marks for each question, and the overall attempt score.';
+$string['marks_help'] = 'The mark obtained for each question and the overall attempt score. You can only select Marks if Maximum marks is selected.';
 $string['match'] = 'Matching';
 $string['matchanswer'] = 'Matching answer';
 $string['matchanswerno'] = 'Matching answer {$a}';
@@ -528,6 +530,9 @@ $string['messageprovider:attempt_grading_complete'] = 'Notification that your at
 $string['messageprovider:submission'] = 'Notification of your students\' quiz submissions';
 $string['max'] = 'Max';
 $string['maxmark'] = 'Maximum mark';
+$string['maxmarks'] = 'Maximum marks';
+$string['maxmarks_help'] = 'The maximum mark available for each question.';
+
 $string['min'] = 'Min';
 $string['minutes'] = 'Minutes';
 $string['missingcorrectanswer'] = 'Correct answer must be specified';
@@ -596,11 +601,11 @@ $string['noquestionsonpage'] = 'Empty page';
 $string['noresponse'] = 'No response';
 $string['noreview'] = 'You are not allowed to review this quiz';
 $string['noreviewattempt'] = 'You are not allowed to review this attempt.';
-$string['noreviewshort'] = 'Not permitted';
+$string['noreviewshort'] = 'Review not permitted';
 $string['noreviewuntil'] = 'You are not allowed to review this quiz until {$a}';
 $string['noreviewuntilshort'] = 'Available {$a}';
 $string['noscript'] = 'JavaScript must be enabled to continue!';
-$string['notavailabletostudents'] = 'Note: This quiz is not currently available to your students';
+$string['notavailabletostudents'] = 'Note: This quiz is currently not available to your students.';
 $string['notenoughrandomquestions'] = 'There are not enough questions in category {$a->category} to create the question {$a->name} ({$a->id}).';
 $string['notenoughsubquestions'] = 'Not enough sub-questions have been defined!<br />Do you want to go back and fix this question?';
 $string['notifyattemptsgradedtask'] = 'Send quiz attempt graded notifications';
@@ -732,6 +737,8 @@ $string['questiondependencyadd'] = 'No restriction on when question {$a->thisq}
 $string['questiondependencyfree'] = 'No restriction on this question';
 $string['questiondependencyremove'] = 'Question {$a->thisq} cannot be attempted until the previous question {$a->previousq} has been completed • Click to change';
 $string['questiondependsonprevious'] = 'This question cannot be attempted until the previous question has been completed.';
+$string['questiondraftonly'] = 'The question {$a} is in draft status. To use it in the quiz, go to the question bank and change the status to ready.';
+$string['questiondraftwillnotwork'] = 'This question is in draft status. To use it in the quiz, go to the question bank and change the status to ready.';
 $string['questioninuse'] = 'The question \'{$a->questionname}\' is currently being used in: <br />{$a->quiznames}<br />The question will not be deleted from these quizzes but only from the category list.';
 $string['questionmissing'] = 'Question for this session is missing';
 $string['questionname'] = 'Question name';
@@ -788,6 +795,7 @@ $string['quizordernotrandom'] = 'Order of quiz not shuffled';
 $string['quizorderrandom'] = '* Order of quiz is shuffled';
 $string['quiz:preview'] = 'Preview quizzes';
 $string['quiz:regrade'] = 'Regrade quiz attempts';
+$string['quiz:reopenattempts'] = 'Reopen never submitted quiz attempts';
 $string['quizreport'] = 'Quiz report';
 $string['quiz:reviewmyattempts'] = 'Review your own attempts';
 $string['quizsettings'] = 'Quiz settings';
@@ -796,13 +804,23 @@ $string['quiz:viewreports'] = 'View quiz reports';
 $string['quiztimer'] = 'Quiz Timer';
 $string['quizwillopen'] = 'This quiz will open {$a}';
 $string['random'] = 'Random question';
+$string['randomcatwithsubcat'] = '{$a} and subcategories';
+$string['randomcoursecatwithsubcat'] = 'Any category inside course category {$a}';
+$string['randomcoursewithsubcat'] = 'Any category in this course';
 $string['randomcreate'] = 'Create random questions';
 $string['randomediting'] = 'Editing a random question';
+$string['randomfaultynosubcat'] = 'Faulty question';
 $string['randomfromcategory'] = 'Random question from category:';
 $string['randomfromexistingcategory'] = 'Random question from an existing category';
 $string['randomfromunavailabletag'] = '{$a} (unavailable)';
+$string['randommodulewithsubcat'] = 'Any category of this quiz';
 $string['randomnumber'] = 'Number of random questions';
 $string['randomnosubcat'] = 'Questions from this category only, not its subcategories.';
+$string['randomqname'] = 'Random question based on filter condition';
+$string['randomqnamecat'] = 'Random ({$a->category}) based on filter condition';
+$string['randomqnamecattags'] = 'Random ({$a->category}) based on filter condition with tags: {$a->tags}';
+$string['randomquestion'] = 'Random question';
+$string['randomqnametags'] = 'Random question based on filter condition with tags: {$a}';
 $string['randomquestion'] = 'Random question';
 $string['randomquestion_help'] = 'A random question is a way of inserting a randomly-chosen question from a specified category or by a specified tag into an activity.';
 $string['randomquestiontags'] = 'Tags';
@@ -810,6 +828,7 @@ $string['randomquestiontags_help'] = 'You can restrict the selection criteria fu
 
 The "random" questions will be selected from the questions that have all these tags.';
 $string['randomquestionusinganewcategory'] = 'Random question using a new category';
+$string['randomsystemwithsubcat'] = 'Any system-level category';
 $string['randomwithsubcat'] = 'Questions from this category and its subcategories.';
 $string['readytosend'] = 'You are about to send your whole quiz to be graded.  Are you sure you want to continue?';
 $string['reattemptquiz'] = 'Re-attempt quiz';
@@ -833,6 +852,13 @@ $string['removepagebreak'] = 'Remove page break';
 $string['removeselected'] = 'Remove selected';
 $string['rename'] = 'Rename';
 $string['renderingserverconnectfailed'] = 'The server {$a} failed to process an RQP request. Check that the URL is correct.';
+$string['reopenattempt'] = 'Reopen';
+$string['reopenattemptareyousuremessage'] = 'This will reopen attempt {$a->attemptnumber} by {$a->attemptuser}.';
+$string['reopenattemptareyousuretitle'] = 'Reopen attempt?';
+$string['reopenattemptwrongstate'] = 'Attempt {$a->attemptid} is in the wrong state ({$a->state}) to be reopened.';
+$string['reopenedattemptwillbeinprogress'] = 'The attempt will remain open and can be continued.';
+$string['reopenedattemptwillbeinprogressuntil'] = 'The attempt will remain open and can be continued until the quiz closes on {$a}.';
+$string['reopenedattemptwillbesubmitted'] = 'The attempt will be immediately submitted for grading.';
 $string['reorderquestions'] = 'Reorder questions';
 $string['reordertool'] = 'Show the reordering tool';
 $string['repaginate'] = 'Repaginate with {$a} questions per page';
@@ -930,6 +956,7 @@ $string['selectedattempts'] = 'Selected attempts...';
 $string['selectmultipleitems'] = 'Select multiple items';
 $string['selectmultipletoolbar'] = 'Select multiple toolbar';
 $string['selectnone'] = 'Deselect all';
+$string['selectquestionslot'] = 'Select question {$a}';
 $string['selectquestiontype'] = '-- Select question type --';
 $string['serveradded'] = 'Server added';
 $string['serveridentifier'] = 'Identifier';
@@ -987,6 +1014,8 @@ $string['stateoverdue'] = 'Overdue';
 $string['stateoverduedetails'] = 'Must be submitted by {$a}';
 $string['status'] = 'Status';
 $string['stoponerror'] = 'Stop on error';
+$string['submission_confirmation'] = 'Submit all your answers and finish?';
+$string['submission_confirmation_unanswered'] = 'Questions without a response: {$a}';
 $string['submitallandfinish'] = 'Submit all and finish';
 $string['subneterror'] = 'Sorry, this quiz has been locked so that it is only accessible from certain locations.  Currently your computer is not one of those allowed to use this quiz.';
 $string['subnetnotice'] = 'This quiz has been locked so that it is only accessible from certain locations. Your computer is not on an allowed subnet. As teacher you are allowed to preview anyway.';
@@ -1025,6 +1054,8 @@ $string['unfinished'] = 'open';
 $string['ungraded'] = 'Ungraded';
 $string['unit'] = 'Unit';
 $string['unknowntype'] = 'Question type not supported at line {$a}. The question will be ignored';
+$string['updatefilterconditon'] = 'Update filter conditions';
+$string['updatefilterconditon_success'] = 'Successfully updated filter conditions';
 $string['updateoverdueattemptstask'] = 'Updating overdue quiz attempts';
 $string['updatesettings'] = 'Update quiz settings';
 $string['updatequizslotswithrandomxofy'] = 'Updating quiz slots with "random" question data ({$a->done}/{$a->total})';
@@ -1057,9 +1088,6 @@ $string['questionversionlatest'] = 'v{$a} (latest)';
 $string['alwayslatest'] = 'Always latest';
 $string['gobacktoquiz'] = 'Go back';
 
-// Deprecated since Moodle 3.11.
-$string['completionattemptsexhausteddesc'] = 'Complete if all available attempts are exhausted';
-
 // Deprecated since Moodle 4.0.
 $string['completionpass'] = 'Require passing grade';
 $string['completionpassdesc'] = 'Student must achieve a passing grade to complete this activity';
@@ -1067,3 +1095,6 @@ $string['completionpass_help'] = 'If enabled, this activity is considered comple
 $string['completiondetail:passgrade'] = 'Receive a pass grade';
 $string['gradetopassnotset'] = 'This quiz does not yet have a grade to pass set. It may be set in the Grade section of the quiz settings.';
 $string['basicideasofquiz'] = 'The basic ideas of quiz-making';
+
+// Deprecated since Moodle 4.3.
+$string['completionminattemptsgroup'] = 'Require attempts';
