@@ -26,6 +26,7 @@
 
 namespace core_user;
 
+use core_external\external_api;
 use core_files_external;
 use core_user_external;
 use externallib_advanced_testcase;
@@ -38,7 +39,13 @@ require_once($CFG->dirroot . '/webservice/tests/helpers.php');
 require_once($CFG->dirroot . '/user/externallib.php');
 require_once($CFG->dirroot . '/files/externallib.php');
 
-class externallib_test extends externallib_advanced_testcase {
+/**
+ * Tests for the user external functions.
+ *
+ * @package core_user
+ * @covers \core_user_external
+ */
+final class externallib_test extends externallib_advanced_testcase {
 
     /**
      * Test get_users
@@ -100,7 +107,7 @@ class externallib_test extends externallib_advanced_testcase {
         $result = core_user_external::get_users($searchparams);
 
         // We need to execute the return values cleaning process to simulate the web service server
-        $result = \external_api::clean_returnvalue(core_user_external::get_users_returns(), $result);
+        $result = external_api::clean_returnvalue(core_user_external::get_users_returns(), $result);
 
         // Check we retrieve the good total number of enrolled users + no error on capability.
         $expectedreturnedusers = 1;
@@ -178,12 +185,30 @@ class externallib_test extends externallib_advanced_testcase {
     /**
      * Test get_users_by_field
      */
-    public function test_get_users_by_field() {
+    public function test_get_users_by_field(): void {
         global $USER, $CFG;
 
         $this->resetAfterTest(true);
 
-        $course = self::getDataGenerator()->create_course();
+        $generator = self::getDataGenerator();
+
+        // Create complex user profile field supporting multi-lang.
+        filter_set_global_state('multilang', TEXTFILTER_ON);
+        filter_set_applies_to_strings('multilang', true);
+
+        $name = '<span lang="en" class="multilang">Employment status</span>'.
+            '<span lang="es" class="multilang">Estado de Empleo</span>';
+        $statuses = 'UE\nSE\n<span lang="en" class="multilang">Other</span><span lang="es" class="multilang">Otro</span>';
+        $generator->create_custom_profile_field(
+            [
+                'datatype' => 'menu',
+                'shortname' => 'employmentstatus',
+                'name' => $name,
+                'param1' => $statuses
+            ]
+        );
+
+        $course = $generator->create_course();
         $user1 = array(
             'username' => 'usernametest1',
             'idnumber' => 'idnumbertest1',
@@ -199,14 +224,16 @@ class externallib_test extends externallib_advanced_testcase {
             'descriptionformat' => FORMAT_MOODLE,
             'city' => 'Perth',
             'country' => 'AU',
+            'profile_field_jobposition' => 'Manager',
+            'profile_field_employmentstatus' => explode('\n', $statuses)[2],
         );
-        $user1 = self::getDataGenerator()->create_user($user1);
+        $user1 = $generator->create_user($user1);
         if (!empty($CFG->usetags)) {
             require_once($CFG->dirroot . '/user/editlib.php');
             $user1->interests = array('Cinema', 'Tennis', 'Dance', 'Guitar', 'Cooking');
             useredit_update_interests($user1, $user1->interests);
         }
-        $user2 = self::getDataGenerator()->create_user(
+        $user2 = $generator->create_user(
                 array('username' => 'usernametest2', 'idnumber' => 'idnumbertest2'));
 
         $generatedusers = array();
@@ -217,9 +244,9 @@ class externallib_test extends externallib_advanced_testcase {
         $roleid = $this->assignUserCapability('moodle/user:viewdetails', $context->id);
 
         // Enrol the users in the course.
-        $this->getDataGenerator()->enrol_user($user1->id, $course->id, $roleid, 'manual');
-        $this->getDataGenerator()->enrol_user($user2->id, $course->id, $roleid, 'manual');
-        $this->getDataGenerator()->enrol_user($USER->id, $course->id, $roleid, 'manual');
+        $generator->enrol_user($user1->id, $course->id, $roleid, 'manual');
+        $generator->enrol_user($user2->id, $course->id, $roleid, 'manual');
+        $generator->enrol_user($USER->id, $course->id, $roleid, 'manual');
 
         // call as admin and receive all possible fields.
         $this->setAdminUser();
@@ -231,7 +258,7 @@ class externallib_test extends externallib_advanced_testcase {
             // Call the external function.
             $returnedusers = core_user_external::get_users_by_field($fieldtosearch,
                         array($USER->{$fieldtosearch}, $user1->{$fieldtosearch}, $user2->{$fieldtosearch}));
-            $returnedusers = \external_api::clean_returnvalue(core_user_external::get_users_by_field_returns(), $returnedusers);
+            $returnedusers = external_api::clean_returnvalue(core_user_external::get_users_by_field_returns(), $returnedusers);
 
             // Expected result differ following the searched field
             // Admin user in the PHPunit framework doesn't have an idnumber.
@@ -289,6 +316,14 @@ class externallib_test extends externallib_advanced_testcase {
                 // Default language and no theme were used for the user.
                 $this->assertEquals($CFG->lang, $returneduser['lang']);
                 $this->assertEmpty($returneduser['theme']);
+
+                if ($returneduser['id'] == $user1->id) {
+                    $this->assertCount(1, $returneduser['customfields']);
+                    $dbvalue = explode('\n', $statuses)[2];
+                    $this->assertEquals($dbvalue, $returneduser['customfields'][0]['value']);
+                    $this->assertEquals('Employment status', $returneduser['customfields'][0]['name']);
+                    $this->assertEquals('Other', $returneduser['customfields'][0]['displayvalue']);
+                }
             }
         }
 
@@ -298,7 +333,7 @@ class externallib_test extends externallib_advanced_testcase {
         // Call the external function.
         $returnedusers = core_user_external::get_users_by_field('username',
                     array($USER->username, $user1->username, $user2->username));
-        $returnedusers = \external_api::clean_returnvalue(core_user_external::get_users_by_field_returns(), $returnedusers);
+        $returnedusers = external_api::clean_returnvalue(core_user_external::get_users_by_field_returns(), $returnedusers);
 
         // Only the own $USER username should be returned
         $this->assertEquals(1, count($returnedusers));
@@ -309,7 +344,7 @@ class externallib_test extends externallib_advanced_testcase {
         // Call the external function.
         $returnedusers = core_user_external::get_users_by_field('username',
             array($USER->username, $user1->username, $user2->username));
-        $returnedusers = \external_api::clean_returnvalue(core_user_external::get_users_by_field_returns(), $returnedusers);
+        $returnedusers = external_api::clean_returnvalue(core_user_external::get_users_by_field_returns(), $returnedusers);
 
         // Only the own $USER username should be returned still.
         $this->assertEquals(1, count($returnedusers));
@@ -322,6 +357,24 @@ class externallib_test extends externallib_advanced_testcase {
 
         $return = new \stdClass();
 
+        $generator = self::getDataGenerator();
+
+        // Create complex user profile field supporting multi-lang.
+        filter_set_global_state('multilang', TEXTFILTER_ON);
+        filter_set_applies_to_strings('multilang', true);
+
+        $name = '<span lang="en" class="multilang">Employment status</span>' .
+            '<span lang="es" class="multilang">Estado de Empleo</span>';
+        $statuses = 'UE\nSE\n<span lang="en" class="multilang">Other</span><span lang="es" class="multilang">Otro</span>';
+        $generator->create_custom_profile_field(
+            [
+                'datatype' => 'menu',
+                'shortname' => 'employmentstatus',
+                'name' => $name,
+                'param1' => $statuses,
+            ]
+        );
+
         // Create the course and fetch its context.
         $return->course = self::getDataGenerator()->create_course();
         $return->user1 = array(
@@ -338,7 +391,8 @@ class externallib_test extends externallib_advanced_testcase {
             'description' => 'This is a description for user 1',
             'descriptionformat' => FORMAT_MOODLE,
             'city' => 'Perth',
-            'country' => 'AU'
+            'country' => 'AU',
+            'profile_field_employmentstatus' => explode('\n', $statuses)[2],
         );
         $return->user1 = self::getDataGenerator()->create_user($return->user1);
         if (!empty($CFG->usetags)) {
@@ -356,6 +410,12 @@ class externallib_test extends externallib_advanced_testcase {
         $this->getDataGenerator()->enrol_user($return->user2->id, $return->course->id, $return->roleid, 'manual');
         $this->getDataGenerator()->enrol_user($USER->id, $return->course->id, $return->roleid, 'manual');
 
+        $group1 = $this->getDataGenerator()->create_group(['courseid' => $return->course->id, 'name' => 'G1']);
+        $group2 = $this->getDataGenerator()->create_group(['courseid' => $return->course->id, 'name' => 'G2']);
+
+        groups_add_member($group1->id, $return->user1->id);
+        groups_add_member($group2->id, $return->user2->id);
+
         return $return;
     }
 
@@ -374,7 +434,7 @@ class externallib_test extends externallib_advanced_testcase {
                     array('userid' => $USER->id, 'courseid' => $data->course->id)));
 
         // We need to execute the return values cleaning process to simulate the web service server.
-        $enrolledusers = \external_api::clean_returnvalue(core_user_external::get_course_user_profiles_returns(), $enrolledusers);
+        $enrolledusers = external_api::clean_returnvalue(core_user_external::get_course_user_profiles_returns(), $enrolledusers);
 
         // Check we retrieve the good total number of enrolled users + no error on capability.
         $this->assertEquals(1, count($enrolledusers));
@@ -398,7 +458,10 @@ class externallib_test extends externallib_advanced_testcase {
             array('userid' => $data->user1->id, 'courseid' => $data->course->id)));
 
         // We need to execute the return values cleaning process to simulate the web service server.
-        $enrolledusers = \external_api::clean_returnvalue(core_user_external::get_course_user_profiles_returns(), $enrolledusers);
+        $enrolledusers = external_api::clean_returnvalue(core_user_external::get_course_user_profiles_returns(), $enrolledusers);
+        // Check we get the requested user and that is in a group.
+        $this->assertCount(1, $enrolledusers);
+        $this->assertCount(1, $enrolledusers[0]['groups']);
 
         foreach($enrolledusers as $enrolleduser) {
             if ($enrolleduser['username'] == $data->user1->username) {
@@ -415,6 +478,11 @@ class externallib_test extends externallib_advanced_testcase {
                 $this->assertEquals(FORMAT_HTML, $enrolleduser['descriptionformat']);
                 $this->assertEquals($data->user1->city, $enrolleduser['city']);
                 $this->assertEquals($data->user1->country, $enrolleduser['country']);
+                // Default language was used for the user.
+                $this->assertEquals($CFG->lang, $enrolleduser['lang']);
+                $this->assertEquals('Employment status', $enrolleduser['customfields'][0]['name']);
+                $this->assertEquals('Other', $enrolleduser['customfields'][0]['displayvalue']);
+
                 if (!empty($CFG->usetags)) {
                     $this->assertEquals(implode(', ', $data->user1->interests), $enrolleduser['interests']);
                 }
@@ -476,7 +544,7 @@ class externallib_test extends externallib_advanced_testcase {
         $createdusers = core_user_external::create_users(array($user1, $user2));
 
         // We need to execute the return values cleaning process to simulate the web service server.
-        $createdusers = \external_api::clean_returnvalue(core_user_external::create_users_returns(), $createdusers);
+        $createdusers = external_api::clean_returnvalue(core_user_external::create_users_returns(), $createdusers);
 
         // Check we retrieve the good total number of created users + no error on capability.
         $this->assertCount(2, $createdusers);
@@ -721,6 +789,7 @@ class externallib_test extends externallib_advanced_testcase {
         global $USER, $CFG, $DB;
 
         $this->resetAfterTest(true);
+        $this->preventResetByRollback();
 
         $wsuser = self::getDataGenerator()->create_user();
         self::setUser($wsuser);
@@ -734,7 +803,7 @@ class externallib_test extends externallib_advanced_testcase {
         // Call the files api to create a file.
         $draftfile = core_files_external::upload($contextid, 'user', 'draft', 0, '/',
                 $filename, $filecontent, null, null);
-        $draftfile = \external_api::clean_returnvalue(core_files_external::upload_returns(), $draftfile);
+        $draftfile = external_api::clean_returnvalue(core_files_external::upload_returns(), $draftfile);
 
         $draftid = $draftfile['itemid'];
 
@@ -786,8 +855,20 @@ class externallib_test extends externallib_advanced_testcase {
         $user4['id'] = $userdeleted->id;
         user_delete_user($userdeleted);
 
+        $user5 = self::getDataGenerator()->create_user();
+        $user5 = array('id' => $user5->id, 'email' => $user5->email);
+
         // Call the external function.
-        core_user_external::update_users(array($user1, $user2, $user3, $user4));
+        $returnvalue = core_user_external::update_users(array($user1, $user2, $user3, $user4));
+        $returnvalue = external_api::clean_returnvalue(core_user_external::update_users_returns(), $returnvalue);
+
+        // Check warnings.
+        $this->assertEquals($user2['id'], $returnvalue['warnings'][0]['itemid']); // Guest user.
+        $this->assertEquals('usernotupdatedguest', $returnvalue['warnings'][0]['warningcode']);
+        $this->assertEquals($user3['id'], $returnvalue['warnings'][1]['itemid']); // Admin user.
+        $this->assertEquals('usernotupdatedadmin', $returnvalue['warnings'][1]['warningcode']);
+        $this->assertEquals($user4['id'], $returnvalue['warnings'][2]['itemid']); // Deleted user.
+        $this->assertEquals('usernotupdateddeleted', $returnvalue['warnings'][2]['warningcode']);
 
         $dbuser2 = $DB->get_record('user', array('id' => $user2['id']));
         $this->assertNotEquals($dbuser2->username, $user2['username']);
@@ -830,6 +911,39 @@ class externallib_test extends externallib_advanced_testcase {
         $dbuserdelpic = $DB->get_record('user', array('id' => $user1['id']));
         $this->assertEquals(0, $dbuserdelpic->picture, 'Picture must be deleted when sent as 0.');
 
+        // Updating user with an invalid email.
+        $user5['email'] = 'bogus';
+        $returnvalue = core_user_external::update_users(array($user5));
+        $returnvalue = external_api::clean_returnvalue(core_user_external::update_users_returns(), $returnvalue);
+        $this->assertEquals('useremailinvalid', $returnvalue['warnings'][0]['warningcode']);
+        $this->assertStringContainsString('Invalid email address',
+            $returnvalue['warnings'][0]['message']);
+
+        // Updating user with a duplicate email.
+        $user5['email'] = $user1['email'];
+        $returnvalue = core_user_external::update_users(array($user1, $user5));
+        $returnvalue = external_api::clean_returnvalue(core_user_external::update_users_returns(), $returnvalue);
+        $this->assertEquals('useremailduplicate', $returnvalue['warnings'][0]['warningcode']);
+        $this->assertStringContainsString('Duplicate email address',
+                $returnvalue['warnings'][0]['message']);
+
+        // Updating a user that does not exist.
+        $user5['id'] = -1;
+        $returnvalue = core_user_external::update_users(array($user5));
+        $returnvalue = external_api::clean_returnvalue(core_user_external::update_users_returns(), $returnvalue);
+        $this->assertEquals('invaliduserid', $returnvalue['warnings'][0]['warningcode']);
+        $this->assertStringContainsString('Invalid user ID',
+                $returnvalue['warnings'][0]['message']);
+
+        // Updating a remote user.
+        $user1['mnethostid'] = 5;
+        user_update_user($user1); // Update user not using webservice.
+        unset($user1['mnethostid']); // The mnet host ID field is not in the allowed field list for the webservice.
+        $returnvalue = core_user_external::update_users(array($user1));
+        $returnvalue = external_api::clean_returnvalue(core_user_external::update_users_returns(), $returnvalue);
+        $this->assertEquals('usernotupdatedremote', $returnvalue['warnings'][0]['warningcode']);
+        $this->assertStringContainsString('User is a remote user',
+                $returnvalue['warnings'][0]['message']);
 
         // Call without required capability.
         $this->unassignUserCapability('moodle/user:update', $context->id, $roleid);
@@ -948,7 +1062,7 @@ class externallib_test extends externallib_advanced_testcase {
         // Call the files api to create a file.
         $draftfile = core_files_external::upload($contextid, $component, $filearea, $itemid, $filepath,
                                                  $filename, $filecontent, $contextlevel, $instanceid);
-        $draftfile = \external_api::clean_returnvalue(core_files_external::upload_returns(), $draftfile);
+        $draftfile = external_api::clean_returnvalue(core_files_external::upload_returns(), $draftfile);
 
         $draftid = $draftfile['itemid'];
         // Make sure the file was created.
@@ -967,6 +1081,55 @@ class externallib_test extends externallib_advanced_testcase {
         $this->assertNotEmpty($file);
     }
 
+
+    /**
+     * Test add_user_private_files quota
+     */
+    public function test_add_user_private_files_quota() {
+        global $USER, $CFG, $DB;
+
+        $this->resetAfterTest(true);
+
+        $context = \context_system::instance();
+        $roleid = $this->assignUserCapability('moodle/user:manageownfiles', $context->id);
+
+        $context = \context_user::instance($USER->id);
+        $contextid = $context->id;
+        $component = "user";
+        $filearea = "draft";
+        $itemid = 0;
+        $filepath = "/";
+        $filename = "Simple.txt";
+        $filecontent = base64_encode("Let us create a nice simple file");
+        $contextlevel = null;
+        $instanceid = null;
+        $browser = get_file_browser();
+
+        // Call the files api to create a file.
+        $draftfile = core_files_external::upload($contextid, $component, $filearea, $itemid, $filepath,
+            $filename, $filecontent, $contextlevel, $instanceid);
+        $draftfile = external_api::clean_returnvalue(core_files_external::upload_returns(), $draftfile);
+        $draftid = $draftfile['itemid'];
+
+        // Call the external function to add the file to private files.
+        core_user_external::add_user_private_files($draftid);
+
+        // Force the quota so we are sure it won't be space to add the new file.
+        $fileareainfo = file_get_file_area_info($contextid, 'user', 'private');
+        $CFG->userquota = $fileareainfo['filesize_without_references'] + 1;
+
+        // Generate a new draftitemid for the same testfile.
+        $draftfile = core_files_external::upload($contextid, $component, $filearea, $itemid, $filepath,
+            $filename, $filecontent, $contextlevel, $instanceid);
+        $draftid = $draftfile['itemid'];
+
+        $this->expectException('moodle_exception');
+        $this->expectExceptionMessage(get_string('maxareabytes', 'error'));
+
+        // Call the external function to include the new file.
+        core_user_external::add_user_private_files($draftid);
+    }
+
     /**
      * Test add user device
      */
@@ -982,7 +1145,8 @@ class externallib_test extends externallib_advanced_testcase {
                 'platform' => 'Android',
                 'version' => '4.2.2',
                 'pushid' => 'apushdkasdfj4835',
-                'uuid' => 'asdnfl348qlksfaasef859'
+                'uuid' => 'asdnfl348qlksfaasef859',
+                'publickey' => null,
                 );
 
         // Call the external function.
@@ -998,14 +1162,15 @@ class externallib_test extends externallib_advanced_testcase {
         $warnings = core_user_external::add_user_device($device['appid'], $device['name'], $device['model'], $device['platform'],
                                                         $device['version'], $device['pushid'], $device['uuid']);
         // We need to execute the return values cleaning process to simulate the web service server.
-        $warnings = \external_api::clean_returnvalue(core_user_external::add_user_device_returns(), $warnings);
+        $warnings = external_api::clean_returnvalue(core_user_external::add_user_device_returns(), $warnings);
         $this->assertCount(1, $warnings);
 
         // Test update an existing device.
         $device['pushid'] = 'different than before';
+        $device['publickey'] = 'MFsxCzAJBgNVBAYTAkZSMRMwEQYDVQQ';
         $warnings = core_user_external::add_user_device($device['appid'], $device['name'], $device['model'], $device['platform'],
-                                                        $device['version'], $device['pushid'], $device['uuid']);
-        $warnings = \external_api::clean_returnvalue(core_user_external::add_user_device_returns(), $warnings);
+            $device['version'], $device['pushid'], $device['uuid'], $device['publickey']);
+        $warnings = external_api::clean_returnvalue(core_user_external::add_user_device_returns(), $warnings);
 
         $this->assertEquals(1, $DB->count_records('user_devices'));
         $updated = $DB->get_record('user_devices', array('pushid' => $device['pushid']));
@@ -1016,7 +1181,7 @@ class externallib_test extends externallib_advanced_testcase {
         $device['pushid'] = 'new different than before';
         $warnings = core_user_external::add_user_device($device['appid'], $device['name'], $device['model'], $device['platform'],
                                                         $device['version'], $device['pushid'], $device['uuid']);
-        $warnings = \external_api::clean_returnvalue(core_user_external::add_user_device_returns(), $warnings);
+        $warnings = external_api::clean_returnvalue(core_user_external::add_user_device_returns(), $warnings);
         $this->assertEquals(2, $DB->count_records('user_devices'));
     }
 
@@ -1054,24 +1219,24 @@ class externallib_test extends externallib_advanced_testcase {
 
         // Try to remove a device that does not exist.
         $result = core_user_external::remove_user_device('1234567890');
-        $result = \external_api::clean_returnvalue(core_user_external::remove_user_device_returns(), $result);
+        $result = external_api::clean_returnvalue(core_user_external::remove_user_device_returns(), $result);
         $this->assertFalse($result['removed']);
         $this->assertCount(1, $result['warnings']);
 
         // Try to remove a device that does not exist for an existing app.
         $result = core_user_external::remove_user_device('1234567890', $device['appid']);
-        $result = \external_api::clean_returnvalue(core_user_external::remove_user_device_returns(), $result);
+        $result = external_api::clean_returnvalue(core_user_external::remove_user_device_returns(), $result);
         $this->assertFalse($result['removed']);
         $this->assertCount(1, $result['warnings']);
 
         // Remove an existing device for an existing app. This will remove one of the two devices.
         $result = core_user_external::remove_user_device($device['uuid'], $device['appid']);
-        $result = \external_api::clean_returnvalue(core_user_external::remove_user_device_returns(), $result);
+        $result = external_api::clean_returnvalue(core_user_external::remove_user_device_returns(), $result);
         $this->assertTrue($result['removed']);
 
         // Remove all the devices. This must remove the remaining device.
         $result = core_user_external::remove_user_device($device['uuid']);
-        $result = \external_api::clean_returnvalue(core_user_external::remove_user_device_returns(), $result);
+        $result = external_api::clean_returnvalue(core_user_external::remove_user_device_returns(), $result);
         $this->assertTrue($result['removed']);
     }
 
@@ -1088,7 +1253,7 @@ class externallib_test extends externallib_advanced_testcase {
         $this->setUser($user);
 
         $result = core_user_external::get_user_preferences();
-        $result = \external_api::clean_returnvalue(core_user_external::get_user_preferences_returns(), $result);
+        $result = external_api::clean_returnvalue(core_user_external::get_user_preferences_returns(), $result);
         $this->assertCount(0, $result['warnings']);
         // Expect 3, _lastloaded is always returned.
         $this->assertCount(3, $result['preferences']);
@@ -1103,14 +1268,14 @@ class externallib_test extends externallib_advanced_testcase {
 
         // Retrieve just one preference.
         $result = core_user_external::get_user_preferences('some_random_text');
-        $result = \external_api::clean_returnvalue(core_user_external::get_user_preferences_returns(), $result);
+        $result = external_api::clean_returnvalue(core_user_external::get_user_preferences_returns(), $result);
         $this->assertCount(0, $result['warnings']);
         $this->assertCount(1, $result['preferences']);
         $this->assertEquals('text', $result['preferences'][0]['value']);
 
         // Retrieve non-existent preference.
         $result = core_user_external::get_user_preferences('non_existent');
-        $result = \external_api::clean_returnvalue(core_user_external::get_user_preferences_returns(), $result);
+        $result = external_api::clean_returnvalue(core_user_external::get_user_preferences_returns(), $result);
         $this->assertCount(0, $result['warnings']);
         $this->assertCount(1, $result['preferences']);
         $this->assertEquals(null, $result['preferences'][0]['value']);
@@ -1118,7 +1283,7 @@ class externallib_test extends externallib_advanced_testcase {
         // Check that as admin we can retrieve all the preferences for any user.
         $this->setAdminUser();
         $result = core_user_external::get_user_preferences('', $user->id);
-        $result = \external_api::clean_returnvalue(core_user_external::get_user_preferences_returns(), $result);
+        $result = external_api::clean_returnvalue(core_user_external::get_user_preferences_returns(), $result);
         $this->assertCount(0, $result['warnings']);
         $this->assertCount(3, $result['preferences']);
 
@@ -1161,7 +1326,7 @@ class externallib_test extends externallib_advanced_testcase {
 
         // Change user profile image.
         $result = core_user_external::update_picture($draftid);
-        $result = \external_api::clean_returnvalue(core_user_external::update_picture_returns(), $result);
+        $result = external_api::clean_returnvalue(core_user_external::update_picture_returns(), $result);
         $picture = $DB->get_field('user', 'picture', array('id' => $user->id));
         // The new revision is in the url for the user.
         $this->assertStringContainsString($picture, $result['profileimageurl']);
@@ -1170,7 +1335,7 @@ class externallib_test extends externallib_advanced_testcase {
 
         // Delete image.
         $result = core_user_external::update_picture(0, true);
-        $result = \external_api::clean_returnvalue(core_user_external::update_picture_returns(), $result);
+        $result = external_api::clean_returnvalue(core_user_external::update_picture_returns(), $result);
         $picture = $DB->get_field('user', 'picture', array('id' => $user->id));
         // No picture.
         $this->assertEquals(0, $picture);
@@ -1184,7 +1349,7 @@ class externallib_test extends externallib_advanced_testcase {
         $draftid = $draftfile['itemid'];
 
         $result = core_user_external::update_picture($draftid, false, $user->id);
-        $result = \external_api::clean_returnvalue(core_user_external::update_picture_returns(), $result);
+        $result = external_api::clean_returnvalue(core_user_external::update_picture_returns(), $result);
         // The new revision is in the url for the user.
         $picture = $DB->get_field('user', 'picture', array('id' => $user->id));
         $this->assertStringContainsString($picture, $result['profileimageurl']);
@@ -1224,20 +1389,20 @@ class externallib_test extends externallib_advanced_testcase {
             ),
             array(
                 'name' => 'htmleditor',
-                'value' => 'tinymce',
+                'value' => 'tiny',
                 'userid' => $user2->id,
             )
         );
 
         $result = core_user_external::set_user_preferences($preferences);
-        $result = \external_api::clean_returnvalue(core_user_external::set_user_preferences_returns(), $result);
+        $result = external_api::clean_returnvalue(core_user_external::set_user_preferences_returns(), $result);
         $this->assertCount(0, $result['warnings']);
         $this->assertCount(2, $result['saved']);
 
         // Get preference from DB to avoid cache.
         $this->assertEquals('atto', $DB->get_field('user_preferences', 'value',
             array('userid' => $user1->id, 'name' => 'htmleditor')));
-        $this->assertEquals('tinymce', $DB->get_field('user_preferences', 'value',
+        $this->assertEquals('tiny', $DB->get_field('user_preferences', 'value',
             array('userid' => $user2->id, 'name' => 'htmleditor')));
     }
 
@@ -1261,7 +1426,7 @@ class externallib_test extends externallib_advanced_testcase {
         );
 
         $result = core_user_external::set_user_preferences($preferences);
-        $result = \external_api::clean_returnvalue(core_user_external::set_user_preferences_returns(), $result);
+        $result = external_api::clean_returnvalue(core_user_external::set_user_preferences_returns(), $result);
         $this->assertCount(1, $result['warnings']);
         $this->assertCount(0, $result['saved']);
         $this->assertEquals('nopermission', $result['warnings'][0]['warningcode']);
@@ -1286,7 +1451,7 @@ class externallib_test extends externallib_advanced_testcase {
         );
 
         $result = core_user_external::set_user_preferences($preferences);
-        $result = \external_api::clean_returnvalue(core_user_external::set_user_preferences_returns(), $result);
+        $result = external_api::clean_returnvalue(core_user_external::set_user_preferences_returns(), $result);
         $this->assertCount(1, $result['warnings']);
         $this->assertCount(0, $result['saved']);
         $this->assertEquals('invaliduser', $result['warnings'][0]['warningcode']);
@@ -1311,7 +1476,7 @@ class externallib_test extends externallib_advanced_testcase {
         );
 
         $result = core_user_external::set_user_preferences($preferences);
-        $result = \external_api::clean_returnvalue(core_user_external::set_user_preferences_returns(), $result);
+        $result = external_api::clean_returnvalue(core_user_external::set_user_preferences_returns(), $result);
         $this->assertCount(0, $result['warnings']);
         $this->assertCount(1, $result['saved']);
         // Cleaned valud of the preference was saved.
@@ -1365,7 +1530,7 @@ class externallib_test extends externallib_advanced_testcase {
         );
 
         $result = core_user_external::set_user_preferences($preferences);
-        $result = \external_api::clean_returnvalue(core_user_external::set_user_preferences_returns(), $result);
+        $result = external_api::clean_returnvalue(core_user_external::set_user_preferences_returns(), $result);
         $this->assertCount(0, $result['warnings']);
         $this->assertCount(1, $result['saved']);
 
@@ -1391,7 +1556,7 @@ class externallib_test extends externallib_advanced_testcase {
 
         // Site policy not set.
         $result = core_user_external::agree_site_policy();
-        $result = \external_api::clean_returnvalue(core_user_external::agree_site_policy_returns(), $result);
+        $result = external_api::clean_returnvalue(core_user_external::agree_site_policy_returns(), $result);
         $this->assertFalse($result['status']);
         $this->assertCount(1, $result['warnings']);
         $this->assertEquals('nositepolicy', $result['warnings'][0]['warningcode']);
@@ -1401,7 +1566,7 @@ class externallib_test extends externallib_advanced_testcase {
         $this->assertEquals(0, $USER->policyagreed);
 
         $result = core_user_external::agree_site_policy();
-        $result = \external_api::clean_returnvalue(core_user_external::agree_site_policy_returns(), $result);
+        $result = external_api::clean_returnvalue(core_user_external::agree_site_policy_returns(), $result);
         $this->assertTrue($result['status']);
         $this->assertCount(0, $result['warnings']);
         $this->assertEquals(1, $USER->policyagreed);
@@ -1409,7 +1574,7 @@ class externallib_test extends externallib_advanced_testcase {
 
         // Try again, we should get a warning.
         $result = core_user_external::agree_site_policy();
-        $result = \external_api::clean_returnvalue(core_user_external::agree_site_policy_returns(), $result);
+        $result = external_api::clean_returnvalue(core_user_external::agree_site_policy_returns(), $result);
         $this->assertFalse($result['status']);
         $this->assertCount(1, $result['warnings']);
         $this->assertEquals('alreadyagreed', $result['warnings'][0]['warningcode']);
@@ -1454,7 +1619,7 @@ class externallib_test extends externallib_advanced_testcase {
 
         // Get my private files information.
         $result = core_user_external::get_private_files_info();
-        $result = \external_api::clean_returnvalue(core_user_external::get_private_files_info_returns(), $result);
+        $result = external_api::clean_returnvalue(core_user_external::get_private_files_info_returns(), $result);
         $this->assertEquals(1, $result['filecount']);
         $this->assertEquals($file->get_filesize(), $result['filesize']);
         $this->assertEquals(0, $result['foldercount']);
@@ -1463,7 +1628,7 @@ class externallib_test extends externallib_advanced_testcase {
         // As admin, get user information.
         $this->setAdminUser();
         $result = core_user_external::get_private_files_info($user->id);
-        $result = \external_api::clean_returnvalue(core_user_external::get_private_files_info_returns(), $result);
+        $result = external_api::clean_returnvalue(core_user_external::get_private_files_info_returns(), $result);
         $this->assertEquals(1, $result['filecount']);
         $this->assertEquals($file->get_filesize(), $result['filesize']);
         $this->assertEquals(0, $result['foldercount']);
@@ -1540,7 +1705,7 @@ class externallib_test extends externallib_advanced_testcase {
         $CFG->maxusersperpage = 3;
 
         $result = \core_user\external\search_identity::execute('Lastt');
-        $result = \external_api::clean_returnvalue(\core_user\external\search_identity::execute_returns(), $result);
+        $result = external_api::clean_returnvalue(\core_user\external\search_identity::execute_returns(), $result);
 
         $this->assertEquals(2, count($result['list']));
         $this->assertEquals(3, $result['maxusersperpage']);
@@ -1557,7 +1722,7 @@ class externallib_test extends externallib_advanced_testcase {
         $CFG->maxusersperpage = 2;
 
         $result = \core_user\external\search_identity::execute('Firstt');
-        $result = \external_api::clean_returnvalue(\core_user\external\search_identity::execute_returns(), $result);
+        $result = external_api::clean_returnvalue(\core_user\external\search_identity::execute_returns(), $result);
 
         $this->assertEquals(2, count($result['list']));
         $this->assertEquals(2, $result['maxusersperpage']);
@@ -1572,7 +1737,7 @@ class externallib_test extends externallib_advanced_testcase {
         $CFG->maxusersperpage = 2;
 
         $result = \core_user\external\search_identity::execute('City One');
-        $result = \external_api::clean_returnvalue(\core_user\external\search_identity::execute_returns(), $result);
+        $result = external_api::clean_returnvalue(\core_user\external\search_identity::execute_returns(), $result);
 
         $this->assertEquals(0, count($result['list']));
         $this->assertEquals(2, $result['maxusersperpage']);
@@ -1587,7 +1752,7 @@ class externallib_test extends externallib_advanced_testcase {
         }
 
         $result = \core_user\external\search_identity::execute('City One');
-        $result = \external_api::clean_returnvalue(\core_user\external\search_identity::execute_returns(), $result);
+        $result = external_api::clean_returnvalue(\core_user\external\search_identity::execute_returns(), $result);
 
         $this->assertEquals(2, count($result['list']));
         $this->assertEquals(2, $result['maxusersperpage']);
@@ -1618,7 +1783,7 @@ class externallib_test extends externallib_advanced_testcase {
             '<ruby>lastname firstname <rp>(</rp><rt>lastnamephonetic firstnamephonetic</rt><rp>)</rp></ruby>';
 
         $result = \core_user\external\search_identity::execute('Ak');
-        $result = \external_api::clean_returnvalue(\core_user\external\search_identity::execute_returns(), $result);
+        $result = external_api::clean_returnvalue(\core_user\external\search_identity::execute_returns(), $result);
 
         $this->assertEquals(1, count($result['list']));
         $this->assertEquals(3, $result['maxusersperpage']);
@@ -1653,7 +1818,7 @@ class externallib_test extends externallib_advanced_testcase {
             ]
         ];
         $result = core_user_external::set_user_preferences($preferences);
-        $result = \external_api::clean_returnvalue(core_user_external::set_user_preferences_returns(), $result);
+        $result = external_api::clean_returnvalue(core_user_external::set_user_preferences_returns(), $result);
         $this->assertCount(1, $result['warnings']);
         $this->assertCount(0, $result['saved']);
 
